==> fetch.php?media=snfit-2.2.1.tar.gz: snfit-2.2.1/src/snphotoz.cc <==
#include <string>
#include <vector>
#include <fstream>

#include "fileutils.h"
#include "fullfit.h"
#include "measureddata.h"
#include "pathutils.h"
#include "instrument.h"
#include "salt2model.h"

using namespace std;

double phasemin=-15;
double phasemax=+45;
double salt2_default_wmin = 3000;
double salt2_default_wmax = 7000;
map< string,vector<string> > fixableparameters;
double zmin = 0;
double zmax = 1.4;



static void usage() {
  cerr << "snphotoz lightcurve1 lightcurve2 ..." << endl;  
  cerr << endl;
  cerr << "options for data selection :"<< endl;
  cerr << "    -r phasemin phasemax     : range of phases  (default is " << phasemin << "," << phasemax << ")"<< endl;
  cerr << "    -w wmin wmax             : wavelength range (default SALT2:"
         << salt2_default_wmin << "," << salt2_default_wmax << ")" << endl;
  cerr << endl;
  cerr << "options for model parameters (need to define the model before): "<< endl;
  cerr << ", for salt2 : ";
  for(size_t i=0;i<fixableparameters["salt2"].size();i++) 
    cerr << " " << fixableparameters["salt2"][i];
  cerr << ")" << endl;
  
  cerr << "    -f name value            : fix the value of parameter 'name'"<< endl;
  cerr << endl;
  cerr << "options for assumed uncertainties : "<< endl;
  cerr << "    -e                       : use (diagonal) model uncertainties in chi2 (for salt2)"<< endl;
  cerr << "    -k                       : use k-correction uncertainties in chi2  (for salt2)"<< endl;
  cerr << "    -U value                 : add observed U-band calibration error"<< endl;
  cerr << "    -z zmin zmax             : range of z to scan (default is " << zmin << " " << zmax << ")" << endl;
  cerr << endl;
  cerr << "See http://supernovae.in2p3.fr/~guy/salt/" << endl;
  cerr << endl;
  

  //cerr << "    -{1,2,3}                 : 1,2 or 3 observed band fitted only"<< endl;
  cerr << "    -E  value                     : error floor to apply for nearby SNe (frac. of max flux)"<< endl;
  //cerr << "    -g                       : fit a galaxy+SN spectrum (default is false)" << endl;
  //cerr << "Debugging options (expert mode): " << endl;
  //cerr << "    -S <band> <wavelenghtshift> : shift the effective filter of a given band by <wavelenghtshift> " << endl;
  exit(EXIT_FAILURE); 
}


void check_param(const string& name , const vector<string>& allowed) {
 
  for(size_t p=0;p<allowed.size();++p) {
    if (name==allowed[p]) return;
  }
  cerr << "cannot deal with unknown parameter " << name << endl;
  usage();
}


#define NBANDS_FOR_SINGLE_FIT 1


#define _GNU_SOURCE 1
#ifndef __USE_GNU
#define __USE_GNU
#endif
#include <fenv.h>

int main(int nargs, char **args)
{
  
  // defaults
  // ------------------------------------------------------
  vector<string> lc;
  int nobservedband = 0;
  float forced_wmin = -100;
  float forced_wmax = -200;
  bool use_model_errors = false;
  bool use_kcorr_errors = false;
  float error_floor_to_apply = 0;
  double uband_calibration_error = 0;
  
  // to crash when NaN
  feenableexcept (FE_INVALID|FE_DIVBYZERO|FE_OVERFLOW);
  

  {
    vector<string> fixableparameters_salt2;
    fixableparameters_salt2.push_back("DayMax");
    fixableparameters_salt2.push_back("Color");
    fixableparameters_salt2.push_back("X0");
    fixableparameters_salt2.push_back("X1");
    fixableparameters_salt2.push_back("Redshift");
    fixableparameters["salt2"]=fixableparameters_salt2;
  }
  
  string band2shift = "none";
  double wavelenghtshift = 0;
  
  map<string,double> fixed_parameter_values;
  map<string,double> initial_parameter_values;
  
  

  // read arguments
  // ------------------------------------------------------
  if (nargs <2) usage();
  for (int i=1; i<nargs; ++i)
    {
      char *arg = args[i];
      if (arg[0] != '-')
	{
	  lc.push_back(arg);continue;
	}
      switch (arg[1])
	{
	case 'r' : phasemin=atof(args[++i]); phasemax=atof(args[++i]); break; 
	case 'w' : forced_wmin = atof(args[++i]); forced_wmax = atof(args[++i]); break;   
	case 'f' :
	  { 
	    string parname = args[++i];
	    double value   = atof(args[++i]);
	    check_param(parname,fixableparameters["salt2"]);
	    cout << "fixing parameter \"" <<  parname << "\"" << endl;
	    fixed_parameter_values[parname]=value;
	    break;
	  }
	  
	case 'e': use_model_errors = true; break;
	case 'k': use_kcorr_errors = true; break;
	case 'U': uband_calibration_error = atof(args[++i]); break;
	case 'E': error_floor_to_apply = atof(args[++i]); break;
	case 'z': zmin =  atof(args[++i]); zmax =  atof(args[++i]); break; 
	default : usage();
	}
    }
  
 
  
  // Load data
  // ------------------------------------------------------
  MeasuredData data(lc,true,band2shift,wavelenghtshift);
  
  if(error_floor_to_apply>0) {
    cout << "applying error floor " << error_floor_to_apply << endl;
    for(size_t b=0;b<data.NBands();b++) {
      data.PointList(b).ApplyErrorFloor(error_floor_to_apply);
    }
  }

    // Dump effective wavelength of filters
  // ------------------------------------------------------
  for(size_t b=0;b<data.NBands();b++) {
    cout << "band " << data.PointList(b).BandName() << " " << data.PointList(b).GetEffectiveFilter().Mean() << endl;
  }
  
  
  // Load model
  // ------------------------------------------------------
  Salt2Model *model=new Salt2Model();
  model->SetWavelengthRange(salt2_default_wmin,salt2_default_wmax);
  
  // Setup of the fitter
  // ------------------------------------------------------
  FullFit lcf(data,*model);
  
  if(nobservedband>0) {
    lcf.SetMaxNumberOfLightcurvesInFit(nobservedband);
    cout << "maximum number of LCs in fit = " << nobservedband << endl;
  }
  
  
  // Fix some parameters
  // ------------------------------------------------------
  if (model->params.LocateParam("X2")) // not used so far 
    model->FixParameter("X2",0.0);
  
  for(map<string,double>::iterator it = fixed_parameter_values.begin(); 
      it!=fixed_parameter_values.end(); ++it) 
    model->FixParameter(it->first,it->second);
  
  if(data.NBands()<3) {
    cout << "Need at least 3 bands for photoz" << endl;
    return EXIT_FAILURE;
  }
  
  
  // Data range used for fit
  // ------------------------------------------------------
  lcf.SetPhaseRange(phasemin,phasemax);
  if(forced_wmax>0) {
    cerr << "forcing wavelength range " << forced_wmin 
	 << " " << forced_wmax << endl;
    model->SetWavelengthRange(forced_wmin,forced_wmax);
  }


  
  // First scan redshift with x1=0 and c=0
  // ------------------------------------------------------ 
  lcf.UseModelErrors(false); 
  lcf.UseKcorrErrors(false);
  lcf.SetUbandSystematicError(false); 
  lcf.UseObserverPhaseWeight(true);
  
  bool color_was_already_fixed = model->params.LocateParam("Color")->isfixed;
  if (! color_was_already_fixed)
    model->FixParameter("Color",0);
  bool x1_was_already_fixed = model->params.LocateParam("X1")->isfixed;
  if (! x1_was_already_fixed)
    model->FixParameter("X1",0);
 
  double redshift_best = lcf.Scan("Redshift",zmin,zmax,int((zmax-zmin)/0.1)+1,false);
  cout << "STEP 1 : redshift = " << redshift_best << endl;
  model->FixParameter("Redshift",redshift_best);

  if (! color_was_already_fixed) {
    LcParam* p_color = model->params.LocateParam("Color");
    p_color->isfixed = false; // release param
    p_color->minv = -10;
    p_color->maxv =  10;
    p_color->step =  0.1;
    
  }
  if (! x1_was_already_fixed) {
    LcParam* p_x1 = model->params.LocateParam("X1");
    p_x1->isfixed = false; // release param
    p_x1->minv = -10;
    p_x1->maxv =  10;
    p_x1->step =  0.1;
  }
  if ( ! lcf.Minimize(true)) { // with init of other params
    cout << "Failed to minimize with z=" << redshift_best << endl;
    return EXIT_FAILURE;
  }
  
  // free redshift
  // -----------------------
  {
    LcParam* p_z = model->params.LocateParam("Redshift");
    p_z->isfixed = false; // release param
    p_z->minv = -0.1;
    p_z->maxv = 12;
    p_z->step = 0.05;
  }
    
  // reset error scheme
  // -----------------------
  lcf.UseModelErrors(use_model_errors); 
  lcf.UseKcorrErrors(use_kcorr_errors); 
  lcf.SetUbandSystematicError(uband_calibration_error);
  lcf.UseObserverPhaseWeight(false);
  
  
  
  // Phase range
  // ------------------------------------------------------  
  lcf.SetPhaseRange(phasemin,phasemax);
  
  
  // --------------------------------------------------
  // Main loop
  // this is a loop cause we have to recompute the model errors
  // according to the previous best fit value
  // as many times as it is necessary
  // --------------------------------------------------
  
  bool continue_loop = true;
  int iteration = -1;
  while(continue_loop) {
    
    
    cout << "ITERATION NUMBER " << ++iteration << endl;
    FitParams input_params = model->params;
    lcf.Minimize(false); // 
      
    continue_loop = false;
    
    // continue to loop if at least one parameter
    // has been modified by more than 1% of its statistical error 
    for(size_t p =0;p<input_params.size();p++) {
      if ( ! input_params[p]->isfixed ) {
	continue_loop |= ( fabs( input_params[p]->val - model->params[p]->val ) > 0.1*model->params[p]->err );
      }
    }
    
    continue_loop &= (iteration<20);
  }
  
  

  lcf.Dump();

  
  // write result
  // ------------------------------------------------------
  string resname = "result_salt2_photoz.dat";
  ofstream res(resname.c_str());
  res << lcf << endl;

  //compute UBV restframe mags and their covariances  
  Instrument standard("STANDARD");
  vector<Filter> rfStuff;
  Vect rfMags;
  Mat rfCov;
  //rfStuff.push_back(standard.EffectiveFilterByBand("U02"));
  rfStuff.push_back(standard.EffectiveFilterByBand("U"));
  rfStuff.push_back(standard.EffectiveFilterByBand("B"));
  rfStuff.push_back(standard.EffectiveFilterByBand("V"));
  FitParams ubv("UBV");
  model->GetRestFrameMagsAtPhase(rfStuff, 0, ubv, "VEGA");
  res << ubv << endl;

  // compute variance and covariance of MB with the other parameters.
  FitParams mb("MB_and_pars");
  model->GetRestFrameMagAndCov(standard.EffectiveFilterByBand("B"),
			       0, mb, "VEGA");

  res << mb <<endl;

  res.close();

  
  //lcf.Scan("W0",-0.6,0.6,100,true); // TEST
  
  delete model;
  
  return EXIT_SUCCESS;
}

==> fetch.php?media=snfit-2.2.2.tar.gz: snfit-2.2.2/src/snphotoz.cc <==
#include <string>
#include <vector>
#include <fstream>

#include "fileutils.h"
#include "fullfit.h"
#include "measureddata.h"
#include "pathutils.h"
#include "instrument.h"
#include "salt2model.h"

using namespace std;

double phasemin=-15;
double phasemax=+45;
double salt2_default_wmin = 3000;
double salt2_default_wmax = 7000;
map< string,vector<string> > fixableparameters;
double zmin = 0;
double zmax = 1.4;



static void usage() {
  cerr << "snphotoz lightcurve1 lightcurve2 ..." << endl;  
  cerr << endl;
  cerr << "options for data selection :"<< endl;
  cerr << "    -r phasemin phasemax     : range of phases  (default is " << phasemin << "," << phasemax << ")"<< endl;
  cerr << "    -w wmin wmax             : wavelength range (default SALT2:"
         << salt2_default_wmin << "," << salt2_default_wmax << ")" << endl;
  cerr << endl;
  cerr << "options for model parameters (need to define the model before): "<< endl;
  cerr << ", for salt2 : ";
  for(size_t i=0;i<fixableparameters["salt2"].size();i++) 
    cerr << " " << fixableparameters["salt2"][i];
  cerr << ")" << endl;
  
  cerr << "    -f name value            : fix the value of parameter 'name'"<< endl;
  cerr << endl;
  cerr << "options for assumed uncertainties : "<< endl;
  cerr << "    -e                       : use (diagonal) model uncertainties in chi2 (for salt2)"<< endl;
  cerr << "    -k                       : use k-correction uncertainties in chi2  (for salt2)"<< endl;
  cerr << "    -U value                 : add observed U-band calibration error"<< endl;
  cerr << "    -z zmin zmax             : range of z to scan (default is " << zmin << " " << zmax << ")" << endl;
  cerr << endl;
  cerr << "See http://supernovae.in2p3.fr/~guy/salt/" << endl;
  cerr << endl;
  

  //cerr << "    -{1,2,3}                 : 1,2 or 3 observed band fitted only"<< endl;
  cerr << "    -E  value                     : error floor to apply for nearby SNe (frac. of max flux)"<< endl;
  //cerr << "    -g                       : fit a galaxy+SN spectrum (default is false)" << endl;
  //cerr << "Debugging options (expert mode): " << endl;
  //cerr << "    -S <band> <wavelenghtshift> : shift the effective filter of a given band by <wavelenghtshift> " << endl;
  exit(EXIT_FAILURE); 
}


void check_param(const string& name , const vector<string>& allowed) {
 
  for(size_t p=0;p<allowed.size();++p) {
    if (name==allowed[p]) return;
  }
  cerr << "cannot deal with unknown parameter " << name << endl;
  usage();
}


#define NBANDS_FOR_SINGLE_FIT 1


#define _GNU_SOURCE 1
#ifndef __USE_GNU
#define __USE_GNU
#endif
#include <fenv.h>

int main(int nargs, char **args)
{
  
  // defaults
  // ------------------------------------------------------
  vector<string> lc;
  int nobservedband = 0;
  float forced_wmin = -100;
  float forced_wmax = -200;
  bool use_model_errors = false;
  bool use_kcorr_errors = false;
  float error_floor_to_apply = 0;
  double uband_calibration_error = 0;
  
  // to crash when NaN
  //feenableexcept (FE_INVALID|FE_DIVBYZERO|FE_OVERFLOW);
  

  {
    vector<string> fixableparameters_salt2;
    fixableparameters_salt2.push_back("DayMax");
    fixableparameters_salt2.push_back("Color");
    fixableparameters_salt2.push_back("X0");
    fixableparameters_salt2.push_back("X1");
    fixableparameters_salt2.push_back("Redshift");
    fixableparameters["salt2"]=fixableparameters_salt2;
  }
  
  string band2shift = "none";
  double wavelenghtshift = 0;
  
  map<string,double> fixed_parameter_values;
  map<string,double> initial_parameter_values;
  
  

  // read arguments
  // ------------------------------------------------------
  if (nargs <2) usage();
  for (int i=1; i<nargs; ++i)
    {
      char *arg = args[i];
      if (arg[0] != '-')
	{
	  lc.push_back(arg);continue;
	}
      switch (arg[1])
	{
	case 'r' : phasemin=atof(args[++i]); phasemax=atof(args[++i]); break; 
	case 'w' : forced_wmin = atof(args[++i]); forced_wmax = atof(args[++i]); break;   
	case 'f' :
	  { 
	    string parname = args[++i];
	    double value   = atof(args[++i]);
	    check_param(parname,fixableparameters["salt2"]);
	    cout << "fixing parameter \"" <<  parname << "\"" << endl;
	    fixed_parameter_values[parname]=value;
	    break;
	  }
	  
	case 'e': use_model_errors = true; break;
	case 'k': use_kcorr_errors = true; break;
	case 'U': uband_calibration_error = atof(args[++i]); break;
	case 'E': error_floor_to_apply = atof(args[++i]); break;
	case 'z': zmin =  atof(args[++i]); zmax =  atof(args[++i]); break; 
	default : usage();
	}
    }
  
 
  
  // Load data
  // ------------------------------------------------------
  MeasuredData data(lc,true,band2shift,wavelenghtshift);
  
  if(error_floor_to_apply>0) {
    cout << "applying error floor " << error_floor_to_apply << endl;
    for(size_t b=0;b<data.NBands();b++) {
      data.PointList(b).ApplyErrorFloor(error_floor_to_apply);
    }
  }

    // Dump effective wavelength of filters
  // ------------------------------------------------------
  for(size_t b=0;b<data.NBands();b++) {
    cout << "band " << data.PointList(b).BandName() << " " << data.PointList(b).GetEffectiveFilter().Mean() << endl;
  }
  
  
  // Load model
  // ------------------------------------------------------
  Salt2Model *model=new Salt2Model();
  model->SetWavelengthRange(salt2_default_wmin,salt2_default_wmax);
  
  // Setup of the fitter
  // ------------------------------------------------------
  FullFit lcf(data,*model);
  
  if(nobservedband>0) {
    lcf.SetMaxNumberOfLightcurvesInFit(nobservedband);
    cout << "maximum number of LCs in fit = " << nobservedband << endl;
  }
  
  
  // Fix some parameters
  // ------------------------------------------------------
  if (model->params.LocateParam("X2")) // not used so far 
    model->FixParameter("X2",0.0);
  
  for(map<string,double>::iterator it = fixed_parameter_values.begin(); 
      it!=fixed_parameter_values.end(); ++it) 
    model->FixParameter(it->first,it->second);
  
  if(data.NBands()<3) {
    cout << "Need at least 3 bands for photoz" << endl;
    return EXIT_FAILURE;
  }
  
  
  // Data range used for fit
  // ------------------------------------------------------
  lcf.SetPhaseRange(phasemin,phasemax);
  if(forced_wmax>0) {
    cerr << "forcing wavelength range " << forced_wmin 
	 << " " << forced_wmax << endl;
    model->SetWavelengthRange(forced_wmin,forced_wmax);
  }


  
  // First scan redshift with x1=0 and c=0
  // ------------------------------------------------------ 
  lcf.UseModelErrors(false); 
  lcf.UseKcorrErrors(false);
  lcf.SetUbandSystematicError(false); 
  lcf.UseObserverPhaseWeight(true);
  
  bool color_was_already_fixed = model->params.LocateParam("Color")->isfixed;
  if (! color_was_already_fixed)
    model->FixParameter("Color",0);
  bool x1_was_already_fixed = model->params.LocateParam("X1")->isfixed;
  if (! x1_was_already_fixed)
    model->FixParameter("X1",0);
 
  double redshift_best = lcf.Scan("Redshift",zmin,zmax,int((zmax-zmin)/0.1)+1,false);
  cout << "STEP 1 : redshift = " << redshift_best << endl;
  model->FixParameter("Redshift",redshift_best);

  if (! color_was_already_fixed) {
    LcParam* p_color = model->params.LocateParam("Color");
    p_color->isfixed = false; // release param
    p_color->minv = -10;
    p_color->maxv =  10;
    p_color->step =  0.1;
    
  }
  if (! x1_was_already_fixed) {
    LcParam* p_x1 = model->params.LocateParam("X1");
    p_x1->isfixed = false; // release param
    p_x1->minv = -10;
    p_x1->maxv =  10;
    p_x1->step =  0.1;
  }
  if ( ! lcf.Minimize(true)) { // with init of other params
    cout << "Failed to minimize with z=" << redshift_best << endl;
    return EXIT_FAILURE;
  }
  
  // free redshift
  // -----------------------
  {
    LcParam* p_z = model->params.LocateParam("Redshift");
    p_z->isfixed = false; // release param
    p_z->minv = -0.1;
    p_z->maxv = 12;
    p_z->step = 0.05;
  }
    
  // reset error scheme
  // -----------------------
  lcf.UseModelErrors(use_model_errors); 
  lcf.UseKcorrErrors(use_kcorr_errors); 
  lcf.SetUbandSystematicError(uband_calibration_error);
  lcf.UseObserverPhaseWeight(false);
  
  
  
  // Phase range
  // ------------------------------------------------------  
  lcf.SetPhaseRange(phasemin,phasemax);
  
  
  // --------------------------------------------------
  // Main loop
  // this is a loop cause we have to recompute the model errors
  // according to the previous best fit value
  // as many times as it is necessary
  // --------------------------------------------------
  
  bool continue_loop = true;
  int iteration = -1;
  while(continue_loop) {
    
    
    cout << "ITERATION NUMBER " << ++iteration << endl;
    FitParams input_params = model->params;
    lcf.Minimize(false); // 
      
    continue_loop = false;
    
    // continue to loop if at least one parameter
    // has been modified by more than 1% of its statistical error 
    for(size_t p =0;p<input_params.size();p++) {
      if ( ! input_params[p]->isfixed ) {
	continue_loop |= ( fabs( input_params[p]->val - model->params[p]->val ) > 0.1*model->params[p]->err );
      }
    }
    
    continue_loop &= (iteration<20);
  }
  
  

  lcf.Dump();

  
  // write result
  // ------------------------------------------------------
  string resname = "result_salt2_photoz.dat";
  ofstream res(resname.c_str());
  res << lcf << endl;

  //compute UBV restframe mags and their covariances  
  Instrument standard("STANDARD");
  vector<Filter> rfStuff;
  Vect rfMags;
  Mat rfCov;
  //rfStuff.push_back(standard.EffectiveFilterByBand("U02"));
  rfStuff.push_back(standard.EffectiveFilterByBand("U"));
  rfStuff.push_back(standard.EffectiveFilterByBand("B"));
  rfStuff.push_back(standard.EffectiveFilterByBand("V"));
  FitParams ubv("UBV");
  model->GetRestFrameMagsAtPhase(rfStuff, 0, ubv, "VEGA");
  res << ubv << endl;

  // compute variance and covariance of MB with the other parameters.
  FitParams mb("MB_and_pars");
  model->GetRestFrameMagAndCov(standard.EffectiveFilterByBand("B"),
			       0, mb, "VEGA");

  res << mb <<endl;

  res.close();

  
  //lcf.Scan("W0",-0.6,0.6,100,true); // TEST
  
  delete model;
  
  return EXIT_SUCCESS;
}
